#ifndef _MOTION_CLIENT_H_
#define _MOTION_CLIENT_H_

#include <pktIO.h>

struct reflex;

/*------------------------------------------------------------*/
/*                     public interface                       */
/*------------------------------------------------------------*/

#if 0
#define AXIS_MODE_DIR 1
#define AXIS_MODE_POS 2
#endif

/*
 * target = What the user wants the values to become.
 * model  = A model of what system should now be.
 * system = The measured state of the real system.
 */

#ifdef __cplusplus
extern "C"
{
#endif
  struct reflexMotion {
    int port;
    
    /* ---------------------------------------- */

    int (*getSystemRpl)    (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis, int pos,
			    int vel, int acc, int trq);
  
    int (*getModelRpl)     (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis, int pos,
			    int vel, int acc, int trq);

    int (*getTargetRpl)    (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis, int pos,
			    int vel, int acc, int trq);

    int (*setLimitsRpl)    (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis);

    int (*getLimitsRpl)    (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis, int trq, int acc,
			    int vel, int velErr, int posErr);

    int (*setPosLimitsRpl) (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis);

    int (*getPosLimitsRpl) (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis,
			    unsigned char enable, int min, int max);

    int (*setDirRpl)       (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis);

    int (*setPosRpl)       (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis);

    int (*setModeRpl)      (struct reflex *reflex, int rv,
			    unsigned char id, int ts,
			    unsigned char axis, unsigned char mode);

    int (*setDefaultsRpl)  (struct reflex *reflex, int rv,
			    unsigned char id, int ts);

    int (*brakeSetRpl)     (struct reflex *reflex, int rv,
			    unsigned char id, int ts);

    int (*brakeReleaseRpl) (struct reflex *reflex, int rv,
			    unsigned char id, int ts);

    int (*setSystemRptRpl) (struct reflex *reflex, int rv,
			    unsigned char id, int ts);

    int (*unknownRpl)      (struct reflex *reflex, unsigned char id,
			    unsigned char size,
			    unsigned char *buffer, unsigned char opcode);

    /* ---------------------------------------- */

    int (*systemRpt)       (struct reflex *reflex, unsigned char id, int ts,
			    unsigned char axis,
			    int pos, int vel, int acc, int trq);
  
    /* ---------------------------------------- */

    int (*getSystemReq)    (struct reflex *reflex, unsigned char id,
			    unsigned char axis);

    int (*getModelReq)     (struct reflex *reflex, unsigned char id,
			    unsigned char axis);
  
    int (*getTargetReq)    (struct reflex *reflex, unsigned char id,
			    unsigned char axis);

    int (*setLimitsReq)    (struct reflex *reflex, unsigned char id,
			    unsigned char axis,
			    int trq, int acc,
			    int vel, int velErr, int posErr);

    int (*getLimitsReq)    (struct reflex *reflex, unsigned char id,
			    unsigned char axis);

    int (*setPosLimitsReq) (struct reflex *reflex, unsigned char id,
			    unsigned char axis,
			    unsigned char enable, int  min, int  max);
  
    int (*getPosLimitsReq) (struct reflex *reflex, unsigned char id,
			    unsigned char axis);

    int (*setDirReq)       (struct reflex *reflex, unsigned char id,
			    unsigned char axis,
			    int vel, int acc, int trq, unsigned char dir);

    int (*setPosReq)       (struct reflex *reflex, unsigned char id,
			    unsigned char axis,
			    int vel, int acc, int trq, int pos);

    int (*setModeReq)      (struct reflex *reflex, unsigned char id,
			    unsigned char axis);

    int (*setDefaultsReq)  (struct reflex *reflex, unsigned char id);

    int (*brakeSetReq)     (struct reflex *reflex, unsigned char id);

    int (*brakeReleaseReq) (struct reflex *reflex, unsigned char id);

    int (*setSystemRptReq) (struct reflex *reflex, unsigned char id,
			    unsigned long period, unsigned long mask);

    /* ---------------------------------------- */

    int (*handlePort)      (struct reflex *reflex, int size,
			    unsigned char *buffer,
			    unsigned char id, unsigned char opcode);

    int (*destroy)         (struct reflexMotion *ptr);
  };

  struct reflexMotion * reflexMotionNew (int port);

#ifdef __cplusplus
}
#endif

/*------------------------------------------------------------*/
/*                    private interface                       */
/*------------------------------------------------------------*/

/*
 * This is not supported.  RWI is not obligated to support,
 * maintain or explain this.  Using these features will
 * probably have unexpected results.
 */

#if 0
#define MOTOR_MODE_PWM 0
#define MOTOR_MODE_TRQ 1

#define AXIS_MODE_PWM 0
#endif

#ifdef __cplusplus
extern "C"
{
#endif

  int motGetNAxesReq        (struct reflex *reflex, unsigned char id);

  int  motGetNAxesRpl        (int len, unsigned char *buffer,
			      int *timeStamp,
			      int *nAxes);

  int motSetGearingReq      (struct reflex *reflex, unsigned char id,
			     int nAxes,
			     int *matrixA, int *matrixB);

  int  motSetGearingRpl      (int len, unsigned char *buffer,
			      int *timeStamp);

  int motGetGearingReq      (struct reflex *reflex, unsigned char id,
			     int nAxes);

  int  motGetGearingRpl      (int len, unsigned char *buffer,
			      int *timeStamp, int *nAxes,
			      int *matrixA, int *matrixB);

  int motMotorSetModeReq    (struct reflex *reflex, unsigned char id,
			     unsigned char motor,
			     int mode);

  int  motMotorSetModeRpl    (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor);

  int motMotorGetModeReq    (struct reflex *reflex, unsigned char id,
			     unsigned char motor);

  int  motMotorGetModeRpl    (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor,
			      int *mode);

  int motMotorSetParmsReq   (struct reflex *reflex, unsigned char id,
			     unsigned char motor,
			     int kTrq, int kVel, int pwmThresh);

  int  motMotorSetParmsRpl   (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor);

  int motMotorGetParmsReq   (struct reflex *reflex, unsigned char id,
			     unsigned char motor);

  int  motMotorGetParmsRpl   (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor,
			      int *kTrq, int *kVel, int *pwmThresh);
  
  int motMotorSetLimitsReq  (struct reflex *reflex, unsigned char id,
			     unsigned char motor,
			     int maxPwm, int maxTrq);

  int  motMotorSetLimitsRpl  (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor);

  int motMotorGetLimitsReq  (struct reflex *reflex, unsigned char id,
			     unsigned char motor);

  int  motMotorGetLimitsRpl  (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor,
			      int *maxPwm, int *maxTrq);

  int motMotorGetDataReq    (struct reflex *reflex, unsigned char id,
			     unsigned char motor);

  int  motMotorGetDataRpl    (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *motor,
			      int *pos, int *vel, int *trq, int *pwm);

  int motAxisSetParmsReq    (struct reflex *reflex, unsigned char id,
			     unsigned char axis,
			     int kD, int kP, int kI, int kDecay);

  int  motAxisSetParmsRpl    (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis);

  int motAxisGetParmsReq    (struct reflex *reflex, unsigned char id,
			     unsigned char axis);

  int  motAxisGetParmsRpl    (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis,
			      int *kD, int *kP, int *kI, int *kDecay);

  int motAxisSetPwmLimitReq (struct reflex *reflex, unsigned char id,
			     unsigned char axis,
			     int pwm);

  int  motAxisSetPwmLimitRpl (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis);

  int motAxisGetPwmLimitReq (struct reflex *reflex, unsigned char id,
			     unsigned char axis);

  int  motAxisGetPwmLimitRpl (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis,
			      int *pwm);

  int motAxisSetPwmReq      (struct reflex *reflex, unsigned char id,
			     unsigned char axis,
			     int pwm);

  int  motAxisSetPwmRpl      (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis);

  int motAxisGetPwmReq      (struct reflex *reflex, unsigned char id,
			     unsigned char axis);

  int  motAxisGetPwmRpl      (int len, unsigned char *buffer,
			      int *timeStamp, unsigned char *axis,
			      int *pwm);

#ifdef __cplusplus
}
#endif

/*============================================================*/

#if 0
#define MOT_AXIS_GET_SYSTEM      0
#define MOT_AXIS_GET_MODEL       1
#define MOT_AXIS_GET_TARGET      2
#define MOT_AXIS_SET_LIMITS      3
#define MOT_AXIS_GET_LIMITS      4
#define MOT_AXIS_SET_POS_LIMITS  5
#define MOT_AXIS_GET_POS_LIMITS  6
#define MOT_AXIS_SET_DIR         7
#define MOT_AXIS_SET_POS         8
#define MOT_AXIS_GET_MODE        9
#define MOT_SET_DEFAULTS        10
#define MOT_BRAKE_SET           11
#define MOT_BRAKE_RELEASE       12

#define MOT_SYSTEM_REPORT       33
#define MOT_SYSTEM_REPORT_REQ   34

#define MOT_GET_NAXES           65
#define MOT_SET_GEARING         66
#define MOT_GET_GEARING         67
#define MOT_MOTOR_SET_MODE      68
#define MOT_MOTOR_GET_MODE      69
#define MOT_MOTOR_SET_PARMS     70
#define MOT_MOTOR_GET_PARMS     71
#define MOT_MOTOR_SET_LIMITS    72
#define MOT_MOTOR_GET_LIMITS    73
#define MOT_MOTOR_GET_DATA      74
#define MOT_AXIS_SET_PARMS      75
#define MOT_AXIS_GET_PARMS      76
#define MOT_AXIS_SET_PWM_LIMIT  77
#define MOT_AXIS_GET_PWM_LIMIT  78
#define MOT_AXIS_SET_PWM        79
#define MOT_AXIS_GET_PWM        80
#endif

#endif
