#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <string.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>

#include "pktIO.h"
#include "motionClient.h"
#include "jstkClient.h"
#include "sonarClient.h"
#include "sysClient.h"

#define TRUE  1
#define FALSE 0
#define ERROR -1

#define LONG_PAUSE 50
#define SHORT_PAUSE 0

#ifndef MAX
#define MAX(x, y) (( (x) > (y) ) ? (x) : (y))
#endif
#ifndef MIN
#define MIN(x, y) (( (x) < (y) ) ? (x) : (y))
#endif

static char filename[256];
static char port[256];
static char nBaudStr[256];
static speed_t nBaud;
static char uBaudStr[256];
static speed_t uBaud;
static driveOn = 0;
static int jx, jy;
static unsigned char jb;
static int tVel, rVel;

static int idSetDir[4];
static int idJstk;
static int idGetSys[2];

static int sonarTimeStamp;

/* ============================================================ */
/* ============================================================ */

/* Port 2 is motion control */

void
handlePort2(struct pktPort *pkt, int size, unsigned char *buffer,
	    unsigned char id, unsigned char opcode) {
  int ii;
  int index;
  int retval;
  int timeStamp;
  unsigned char axis, port;
  int pos, vel, acc, trq;

  index = 0;
  port = 2;

  switch (opcode) {
  case MOT_SYSTEM_REPORT:
    retval = motAxisGetSystemRpl(size, buffer,
				 &timeStamp, &axis,
				 &pos, &vel, &acc, &trq);
    if (axis == 0) {
      tVel = vel;
    }
    else if (axis == 1) {
      rVel = vel;
    }

    fprintf(stderr, "MOT_SYSTEM_REPORT:\n\tretval = %d\n", retval);
    fprintf(stderr, "\tid        = %d\n", id);
    fprintf(stderr, "\ttimeStamp = %d\n", timeStamp);
    fprintf(stderr, "\taxis      = %d\n", axis);
    fprintf(stderr, "\tpos       = %d\n", pos);
    fprintf(stderr, "\tvel       = %d\n", vel);
    fprintf(stderr, "\tacc       = %d\n", acc);
    fprintf(stderr, "\ttrq       = %d\n", trq);

    break;

  case MOT_AXIS_GET_SYSTEM:
    retval = motAxisGetSystemRpl(size, buffer,
				 &timeStamp, &axis,
				 &pos, &vel, &acc, &trq);
    if (axis == 0) {
      tVel = vel;
    }
    else if (axis == 1) {
      rVel = vel;
    }

    if (id == 255) {
      fprintf(stderr, "MOT_AXIS_GET_SYSTEM:\n\tretval = %d\n", retval);
      fprintf(stderr, "\tid        = %d\n", id);
      fprintf(stderr, "\ttimeStamp = %d\n", timeStamp);
      fprintf(stderr, "\taxis      = %d\n", axis);
      fprintf(stderr, "\tpos       = %d\n", pos);
      fprintf(stderr, "\tvel       = %d\n", vel);
      fprintf(stderr, "\tacc       = %d\n", acc);
      fprintf(stderr, "\ttrq       = %d\n", trq);
    }

    idGetSys[axis] = id;

    break;

  case MOT_AXIS_GET_MODEL:
    retval = motAxisGetModelRpl(size, buffer,
				 &timeStamp, &axis,
				 &pos, &vel, &acc, &trq);
    fprintf(stderr, "MOT_AXIS_GET_MODEL:\n\tretval = %d\n", retval);
    fprintf(stderr, "\tid        = %d\n", id);
    fprintf(stderr, "\ttimeStamp = %d\n", timeStamp);
    fprintf(stderr, "\taxis      = %d\n", axis);
    fprintf(stderr, "\tpos       = %d\n", pos);
    fprintf(stderr, "\tvel       = %d\n", vel);
    fprintf(stderr, "\tacc       = %d\n", acc);
    fprintf(stderr, "\ttrq       = %d\n", trq);
    break;

  case MOT_AXIS_GET_TARGET:
    retval = motAxisGetTargetRpl(size, buffer,
				 &timeStamp, &axis,
				 &pos, &vel, &acc, &trq);
    fprintf(stderr, "MOT_AXIS_GET_TARGET:\n\tretval = %d\n", retval);
    fprintf(stderr, "\tid        = %d\n", id);
    fprintf(stderr, "\ttimeStamp = %d\n", timeStamp);
    fprintf(stderr, "\taxis      = %d\n", axis);
    fprintf(stderr, "\tpos       = %d\n", pos);
    fprintf(stderr, "\tvel       = %d\n", vel);
    fprintf(stderr, "\tacc       = %d\n", acc);
    fprintf(stderr, "\ttrq       = %d\n", trq);
    break;

  case MOT_AXIS_SET_DIR:
    retval = motAxisSetDirRpl(size, buffer, &timeStamp, &axis);
#if 0
    fprintf(stderr, "MOT_AXIS_SET_DIR:  retval=%d id=%d ts=%d axis=%d\n",
	    retval, id, timeStamp, axis);
#endif
    idSetDir[axis] = id;

    break;

  default:
    fprintf(stderr, "%s:%6d:%s() - ", __FILE__, __LINE__, __FUNCTION__);
    for (ii=0; ii<size; ii++) {
      fprintf(stderr, " 0x%02X", ((unsigned)buffer[ii]));
    }
    fprintf(stderr, "\n");
  }
}

/* ============================================================ */

/* Port 3 is jstk */

void
handlePort3(struct pktPort *pkt, int size, unsigned char *buffer,
	    unsigned char id, unsigned char opcode) {
  int ii;
  int index;
  int timeStamp;

  index = 0;

  switch (opcode) {
  case JSTK_GET_STATE:
    jstkGetStateRpl(size, buffer,
		    &timeStamp,
		    &jx, &jy, &jb);

    if (id == 255) {
      fprintf(stderr, "JSTK_GET_STATE:\n");
      fprintf(stderr, "\tid        = %d\n", id);
      fprintf(stderr, "\ttimeStamp = %d\n", timeStamp);
      fprintf(stderr, "\tx         = %d\n", jx);
      fprintf(stderr, "\ty         = %d\n", jy);
      fprintf(stderr, "\tbuttons   = %d\n", jb);
    }

    idJstk = id;

    break;

  default:
    fprintf(stderr, "read: port3 : id=%d opcode=%d : ", id, opcode);
    for (ii=0; ii<size; ii++) {
      fprintf(stderr, " 0x%02X", ((unsigned)buffer[ii]));
    }
    fprintf(stderr, "\n");
  }
}

/* ============================================================ */

/* Port 4 is sonar */

void
handlePort4(struct pktPort *pkt, int size, unsigned char *buffer,
	    unsigned char id, unsigned char opcode) {
  int ii;
  int index;
  int retval;
  int timeStamp;
  unsigned char sid[256];
  unsigned short svalue[256];
  int count;

  index = 0;

  switch (opcode) {
  case SONAR_RUN:
    retval = sonarRunRpl(size, buffer,
			 &timeStamp);
    fprintf(stderr, "SONAR_RUN: retval = %d\n", retval);
    break;

  case SONAR_GET_UPDATE:
    count = 60;
    retval = sonarGetUpdateRpl(size, buffer, &sonarTimeStamp,
			       &count, sid, svalue);

#if 1
    fprintf(stderr, "SONAR_GET_UPDATE: count=%d timeStamp = %d\n",
	    count, sonarTimeStamp);
#endif

    for (ii=0; ii<count; ii++) {
      fprintf(stderr, "   SONAR %02X: %5d\n", sid[ii], svalue[ii]*597/542-11);
    }

    fprintf(stderr, "\n");
    break;

  case SONAR_REPORT:
    count = 60;
    retval = sonarGetUpdateRpl(size, buffer, &sonarTimeStamp,
			       &count, sid, svalue);

#if 1
    fprintf(stderr, "SONAR_REPORT: count=%d timeStamp = %d\n",
	    count, sonarTimeStamp);
#endif

    for (ii=0; ii<count; ii++) {
      fprintf(stderr, "   SONAR %02X: %5d\n", sid[ii], svalue[ii]*597/542-11);
    }

    fprintf(stderr, "\n");
    break;

  default:
    fprintf(stderr, "read: port3 : id=%d opcode=%d : ", id, opcode);
    for (ii=0; ii<size; ii++) {
      fprintf(stderr, " 0x%02X", ((unsigned)buffer[ii]));
    }
    fprintf(stderr, "\n");
  }
}

/* ============================================================ */

/* default port handler */

void
defaultPortHandler(struct pktPort *pkt, int size, unsigned char *buffer,
		   unsigned char port, unsigned char id, unsigned char opcode)
{
  int ii;

  fprintf(stderr, "read: port=%d :", port);
  for (ii=0; ii<size; ii++) {
    fprintf(stderr, " 0x%02X", ((unsigned)buffer[ii]));
  }
  fprintf(stderr, "\n");
}

/* ============================================================ */

/* Port 0 is unpacketed data */

void
handlePort0(struct pktPort *pkt, int size, unsigned char *buffer, 
	    unsigned char id, unsigned char opcode)
{
  static int state = 1;  /* all this static stuff is A Bad Thing (tm) */
  static FILE *app;
  static int count;
  static time_t start;
  static char lbuffer[1024];
  time_t stop;
  int fd;
  int ii;
  unsigned char port;

  fd = pktGetFd(pkt);

  if (size>0) {
    buffer[size] = 0;
    fprintf(stderr, "%s", buffer); /* So far we always want to echo data */
  }
}

/* ============================================================ */
/* ============================================================ */

static void readLoop(struct pktPort *pkt)
{
  int fd;
  char buffer[256];
  int nread;
  unsigned char port, id, opcode;
  fd_set fds;
  struct timeval timeout;

  fd = pktGetFd(pkt);

  do {
    timeout.tv_sec = 0;
    timeout.tv_usec = 100000;
    FD_ZERO(&fds);
    FD_SET(STDIN_FILENO, &fds);
    FD_SET(fd, &fds);
    select(fd+1, &fds, NULL, NULL, &timeout);

    if (FD_ISSET(fd, &fds)) {
      do {
	nread = pktRead(pkt, buffer, &port, &id, &opcode);

	if (nread < 0) {
	  fflush(stdout);
	  fprintf(stderr, "[%s:%6d:%s() - nread = %d]\n",
		  __FILE__, __LINE__, __FUNCTION__, nread);
	  continue;
	}

	if (nread > 0) {
	  switch (port) {
	  case 0:  handlePort0(pkt, nread, buffer, id, opcode); break;
	  case 1:  handlePort1(pkt, nread, buffer, id, opcode); break;
	  case 2:  handlePort2(pkt, nread, buffer, id, opcode); break;
	  case 3:  handlePort3(pkt, nread, buffer, id, opcode); break;
	  case 4:  handlePort4(pkt, nread, buffer, id, opcode); break;
	  default:  defaultPortHandler(pkt, nread, buffer, port, id, opcode);
	  }
	  handlePort0(pkt, -3, buffer, 255, 255);
	}
      } while (nread != 0);
    }
    else if (FD_ISSET(STDIN_FILENO, &fds)) { /* copy from stdin to port */
      nread = read(STDIN_FILENO, buffer, 255);
      buffer[nread] = 0;
      handleStdin(pkt, nread, buffer);
    }
    else {
      handlePort0(pkt, 0, buffer, 255, 255);
    }
  } while (1);
}
